#!/bin/bash

# this script generates convolve files for 90 degree phase
# shift using 90.orc and 90.sco

# generate a single convolve file

function	cvgen ()
{
	CVNUM="${1}"	# convolve file number
	SRATE="${2}"	# sample rate
	IRLEN="${3}"	# total length of impulse response
	FLSTR="${4}"	# start position
	FLLEN="${5}"	# file length

	# create temporary file with settings

	rm -f "/tmp/__90_ir_params_.orc"

	echo -e ""				>  "/tmp/__90_ir_params_.orc"
	echo -e "#define SRATE # $SRATE #"	>> "/tmp/__90_ir_params_.orc"
	echo -e "#define IR_LENGTH # $IRLEN #"	>> "/tmp/__90_ir_params_.orc"
	echo -e "#define FL_START # $FLSTR #"	>> "/tmp/__90_ir_params_.orc"
	echo -e "#define FL_LENGTH # $FLLEN #"	>> "/tmp/__90_ir_params_.orc"
	echo -e ""				>> "/tmp/__90_ir_params_.orc"

	# render IR file

	rm -f "90.sf" "convolve.${CVNUM}"
	csound -d -m0 -H0 -f -J -o "90.sf" "90.orc" "90.sco"		\
		> "/dev/null" 2>&1 ||					\
		{ echo -e "*** Error running Csound" ; exit -1 ; }
	csound -U cvanal "90.sf" "convolve.${CVNUM}" > "/dev/null" 2>&1
	if ( ! ( test -s "convolve.${CVNUM}" ) ) ; then
		echo -e "*** Error running cvanal" ; exit -1 ;
	fi

	# delete temporary files

	rm -f "/tmp/__90_ir_params_.orc" "90.sf"
}

# render all impulse responses

cvgen 1980  24000  620    0  128
cvgen 1981  24000  620  128  256
cvgen 1982  24000  620  384  256

cvgen 1983  48000 1240    0  256
cvgen 1984  48000 1240  256  512
cvgen 1985  48000 1240  768  512

cvgen 1986  96000 2480    0  512
cvgen 1987  96000 2480  512 1024
cvgen 1988  96000 2480 1536 1024

cvgen 1989 192000 4960    0 1024
cvgen 1990 192000 4960 1024 2048
cvgen 1991 192000 4960 3072 2048

